import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.math.*;


/**This class retrieves the the polytopes from the 4 partitions:
-graph PET partition
-plaid PET partition
-graph triple partition 
-plaid triple partition
All polytopes are scaled up by 60 to clear denominators. */

public class AllPolytopesLong {

    public static PolytopeLong graph(int k) {
	int[][] P1=DataGraphPolytopes.data(k);
	PolytopeLong X=new PolytopeLong();
	X.count=P1.length;
	for(int i=0;i<P1.length;++i) {
	    X.V[i]=new Vector4Long(60*P1[i][0],60*P1[i][1],60*P1[i][2],60*P1[i][3]);
	}
	int[] type=DataGraphPolytopes.type(k);
	X.type[1]=type[0];
	X.type[2]=type[1];
	return(X);
    }


    /**This routine works with the quadruple indices listed in the
       DataTripleIndex file.  It gets the polytope and then appropriately
       applies the lattice group action.*/

    public static PolytopeLong graph(int[] c) {
	PolytopeLong Q=graph(c[3]); 
	Q=graphShift(c[0],c[1],c[2],Q); 
	return Q;
    }

    public static PolytopeLong plaid(int k) {
	int[][] P1=DataPlaidPolytopes.data(k);
	PolytopeLong X=new PolytopeLong();
	X.count=P1.length;
	for(int i=0;i<P1.length;++i) {
	    X.V[i]=new Vector4Long(60*P1[i][0],60*P1[i][1],60*P1[i][2],60*P1[i][3]);
	}
	int[] type=DataPlaidPolytopes.type(k);
	X.type[1]=type[0];
	X.type[2]=type[1];
	return(X);
    }

    public static PolytopeLong graphDouble(int k) {
	int[][] P1=DataGraphDoubles.data(k);
	PolytopeLong X=new PolytopeLong();
	X.count=P1.length;
	for(int i=0;i<P1.length;++i) {
	    X.V[i]=new Vector4Long(60*P1[i][0],60*P1[i][1],60*P1[i][2],60*P1[i][3]);
	}
	return(X);
    }

    public static PolytopeLong graphTriple(int k) {
	int[][] P1=DataGraphTriples.data(k);
	PolytopeLong X=new PolytopeLong();
	X.count=P1.length;
	for(int i=0;i<P1.length;++i) {
	    X.V[i]=new Vector4Long(P1[i][0],P1[i][1],P1[i][2],P1[i][3]);
	}
	int[][] Q0=DataTripleCodes.data();
	int[] Q1=Q0[k];
	for(int i=0;i<6;++i) X.code[i]=Q1[i];
	return(X);
    }



    public static PolytopeLong plaidBarrier(int k) {
	int[][] P1=DataPlaidBarriers.data(k);
	PolytopeLong X=new PolytopeLong();
	X.count=P1.length;
	for(int i=0;i<P1.length;++i) {
	    X.V[i]=new Vector4Long(60*P1[i][0],60*P1[i][1],60*P1[i][2],60*P1[i][3]);
	}
	return(X);
    }



    public static PolytopeLong plaidTriple(int k) {
	int[][] P1=DataPlaidTriples.data(k);
	PolytopeLong X=new PolytopeLong();
	X.count=P1.length;
	for(int i=0;i<P1.length;++i) {
	    X.V[i]=new Vector4Long(P1[i][0],P1[i][1],P1[i][2],P1[i][3]);
	}
	int[][] Q0=DataTripleCodes.data();
	int[] Q1=Q0[k];
	for(int i=0;i<6;++i) X.code[i]=Q1[i];
	return(X);
    }




    /**These are just like what we do for the real polytopes but here the maps
       are conjugated by 60-dilation.  This is the action of the graph PET
       lattice on the graph PET polytopes. */

    public static PolytopeLong graphShift(int q1,int q2,int q3,PolytopeLong P) {
	PolytopeLong Q=new PolytopeLong(P);
	Q=graphShift1(q1,Q);
	Q=graphShift2(q2,Q);
	Q=graphShift3(q3,Q);
	for(int i=0;i<6;++i) Q.code[i]=P.code[i];
	return(Q);
    }

    public static PolytopeLong graphShift1(int m,PolytopeLong P) {
	PolytopeLong Q=new PolytopeLong();
	Q.count=P.count;
	for(int i=0;i<P.count;++i) {
	    Q.V[i]=Vector4Long.plus(P.V[i],new Vector4Long(60*m,-60*m,-60*m,0));
	}
	Q.type[0]=P.type[0];
	Q.type[1]=P.type[1];
	Q.type[2]=P.type[2];
	return(Q);
    }


    public static PolytopeLong graphShift2(int m,PolytopeLong P) {
	PolytopeLong Q=new PolytopeLong();
	Q.count=P.count;
	for(int i=0;i<P.count;++i) {
	    Q.V[i]=Vector4Long.plus(P.V[i],new Vector4Long(0,m*P.V[i].x[3]+60*m,-m*P.V[i].x[3]+60*m,0));
	}
	Q.type[0]=P.type[0];
	Q.type[1]=P.type[1];
	Q.type[2]=P.type[2];
	return(Q);
    }

    public static PolytopeLong graphShift3(int m,PolytopeLong P) {
	PolytopeLong Q=new PolytopeLong();
	Q.count=P.count;
	for(int i=0;i<P.count;++i) {
	    Q.V[i]=Vector4Long.plus(P.V[i],new Vector4Long(0,0,m*P.V[i].x[3]+60*m,0));
	}
	Q.type[0]=P.type[0];
	Q.type[1]=P.type[1];
	Q.type[2]=P.type[2];
	return(Q);
    }



    /**Here is the action of the plaid PET lattice on the plaid PET
       polytopes. Again, the original maps are conjugated by a factor
       of 60, to clear denominators.*/


    public static PolytopeLong plaidShift(int a1,int a2,int a3,PolytopeLong X) {
	PolytopeLong Y=new PolytopeLong();
	Y.count=X.count;
	for(int i=0;i<X.count;++i) {
	    Y.V[i]=plaidShift(a1,a2,a3,X.V[i]);
	}

	if((10+a1)%2==1) {
	  Y.type[1]=X.type[2];
  	  Y.type[2]=X.type[1];
	}
	else {
	  Y.type[1]=X.type[1];
  	  Y.type[2]=X.type[2];
	}
	return(Y);
    }

    public static Vector4Long plaidShift(int a1,int a2,int a3,Vector4Long V) {
	Vector4Long W=new Vector4Long();
	long x=V.x[0];
	long y=V.x[1];
	long z=V.x[2];
	long P=V.x[3];
	W.x[0]=120*a1+x;
	W.x[1]=P*a1+120*a2+y;
	W.x[2]=P*a1+120*a3+z;
	W.x[3]=P;
	return(W);
    }


    /**Here is the plaid curve following map*/

    public static PolytopeLong plaidMap(int direction,PolytopeLong X) {
	if(direction==0) return(X);
	if(X.type[1]==X.type[2]) return(X);
	PolytopeLong Y=new PolytopeLong();
	Y.count=X.count;
	int k=X.type[direction];
	for(int i=0;i<Y.count;++i) {
	    Y.V[i]=plaidMap(k,X.V[i]);
	}
	Y.type[1]=X.type[1];
	Y.type[2]=X.type[2];
	return(Y);
    }

    public static Vector4Long plaidMap(int k,Vector4Long V) {
	if(k==0) return plaidMap0(V);
	if(k==1) return plaidMap1(V);
	if(k==2) return plaidMap2(V);
	if(k==3) return plaidMap3(V);
	return null;
    }

    public static Vector4Long plaidMap0(Vector4Long V) {
	Vector4Long W=new Vector4Long();
	W.x[0]=V.x[0]-120;
	W.x[1]=V.x[1];
	W.x[2]=V.x[2]-2*V.x[3];	
        W.x[3]=V.x[3];
	return(W);
    }

    public static Vector4Long plaidMap1(Vector4Long V) {
	Vector4Long W=new Vector4Long();
	W.x[0]=V.x[0]-2*V.x[3];
	W.x[1]=V.x[1]-2*V.x[3];
	W.x[2]=V.x[2]-2*V.x[3];	
        W.x[3]=V.x[3];
	return(W);
    }

    public static Vector4Long plaidMap2(Vector4Long V) {
	Vector4Long W=new Vector4Long();
	W.x[0]=V.x[0]+120;
	W.x[1]=V.x[1];
	W.x[2]=V.x[2]+2*V.x[3];
	W.x[3]=V.x[3];
	return(W);
    }

    public static Vector4Long plaidMap3(Vector4Long V) {
	Vector4Long W=new Vector4Long();
	W.x[0]=V.x[0]+2*V.x[3];
	W.x[1]=V.x[1]+2*V.x[3];
	W.x[2]=V.x[2]+2*V.x[3];
 	W.x[3]=V.x[3];
	return(W);
    }







}
