import java.applet.Applet;
import java.awt.event.*;
import java.awt.*;
import java.awt.geom.*;


/**This is wrapper class for a list of points in the
   plane.  The main routines in this class are 
   1. taking the convex hull of a finite set of points.
   2. intersecting two convex polygons.
**/


public class PolygonWrapper {
    Complex[] z=new Complex[500];
    int count;
    int edge;
    int t1,t2;

    /**Constructors**/
    public PolygonWrapper() {}

    public PolygonWrapper(PolygonWrapper P) {
	for(int i=0;i<P.count;++i) z[i]=new Complex(P.z[i]);
	edge=P.edge;
	count=P.count;
	t1=P.t1;
	t2=P.t2;
    }


    public PolygonWrapper(int cc,Complex[] zz) {
	this.count=cc;
	for(int i=0;i<cc;++i) {
             z[i]=new Complex(zz[i]);
	}
    }


    /**converting to a path*/

    public Path2D.Double toPath2D() {
	Path2D.Double gp=new Path2D.Double();
	gp.moveTo(z[0].x,z[0].y);
	for(int i=0;i<count;++i) {
	    gp.lineTo(z[i].x,z[i].y);
	}
	gp.closePath();
	return(gp);
    }


    /**printout**/

    public void print() {
	System.out.println("count "+count);
	for(int i=0;i<count;++i) z[i].print();
    }

    /**center and slight shrinking*/

    public Complex center() {
	Complex w=new Complex();
	for(int i=0;i<count;++i) {
	    w=Complex.plus(w,this.z[i]);
	}
	w.x=w.x/count;
	w.y=w.y/count;
	return(w);
    }

    public PolygonWrapper shrink(double t) {
	Complex c=center();
	PolygonWrapper P=new PolygonWrapper();
	P.count=count;
	for(int i=0;i<P.count;++i) {
	    double x=t*c.x+(1-t)*z[i].x;
	    double y=t*c.y+(1-t)*z[i].y;
	    P.z[i]=new Complex(x,y);
	}
	return(P);
    }

    /**This takes care of an irritating special case*/

    public static PolygonWrapper specialIntersect(PolygonWrapper P1,PolygonWrapper P2) {
	Path2D.Double gp=P2.toPath2D();
	if(gp.contains(P1.z[0].x,P1.z[0].y)==true) return P1;
	return null;
    }

    /**This is the main intersection routine*/

    public static PolygonWrapper intersectX(PolygonWrapper P1,PolygonWrapper P2) {
	if(P1==null) return(null);
	if(P2==null) return(null);
	if(P1.count==1) return specialIntersect(P1,P2);
	if(P2.count==1) return specialIntersect(P2,P1);

	PolygonWrapper Q=new PolygonWrapper();
	Q.count=0;
	for(int i=0;i<P1.count;++i) {
	    for(int j=0;j<P2.count;++j) {
		int i1=i;
		int i2=(i+1)%P1.count;
		int j1=j;
		int j2=(j+1)%P2.count;
		Complex q=intersect(P1.z[i1],P1.z[i2],P2.z[j1],P2.z[j2]);
		if(q!=null) {
		    if(badNumber(q)==false) {
		    Q.z[Q.count]=new Complex(q);
		    ++Q.count;
		    }
		}
	    }
	}

	for(int i=0;i<P1.count;++i) {
	    if(contains(P2,P1.z[i])==true) { 
               Q.z[Q.count]=new Complex(P1.z[i]);
	       ++Q.count;
	    }
	}

	for(int i=0;i<P2.count;++i) {
	    if(contains(P1,P2.z[i])==true) { 
               Q.z[Q.count]=new Complex(P2.z[i]);
	       ++Q.count;
	    }
	}

	try{Q=cheapHull(Q);}
	catch(Exception e) {return(null);}
	return(Q);
    }

    public static boolean badNumber(Complex q) {
	if(badNumber(q.x)==true) return(true);
	if(badNumber(q.y)==true) return(true);
	return(false);
    }


    public static boolean badNumber(double d) {
	double D=Double.valueOf(d);
	if(Double.isNaN(D)==true) return(true);
	if(Double.isInfinite(D)==true) return(true);
	return(false);
    }

    public static boolean contains(PolygonWrapper P,Complex z) {
	for(int i=0;i<P.count;++i) {
	    int i1=i;
	    int i2=(i+1)%P.count;
	    Complex w1=P.z[i1];
	    Complex w2=P.z[i2];
	    if(Complex.dist(z,w1)<.0000001) return(true);
	    boolean test=Complex.isPositivelyOriented(z,w1,w2);
	    if(test==false) return(false);
	}
	return(true);
    }


    public static boolean checkSegment(Complex z1,Complex w1,Complex w2) {
	double d1=Complex.dist(z1,w1);
	double d2=Complex.dist(z1,w2);
	double d3=Complex.dist(w1,w2);
	double test=Math.abs(d1+d2-d3);
	if(test<.0000000001) return(true);
	return(false);
    }


    public static Complex intersect(Complex z1,Complex z2,Complex w1,Complex w2) {
	if(checkSegment(z1,w1,w2)==true) return(z1);
	if(checkSegment(z2,w1,w2)==true) return(z2);
	if(checkSegment(w1,z1,z2)==true) return(w1);
	if(checkSegment(w2,z1,z2)==true) return(w2);
	Complex q=Complex.findCross(z1,z2,w1,w2);
	double d1=Math.max(Complex.dist(q,z1),Complex.dist(q,z2));
	double d2=Complex.dist(z1,z2);
	if(d1>d2) return(null);
        d1=Math.max(Complex.dist(q,w1),Complex.dist(q,w2));
	d2=Complex.dist(w1,w2);
	if(d1>d2) return(null);
	return(q);
    }

    /**this routine finds the lowest point of
   a polygon.  if there are more than one
   of these points, the routine picks the
   last one.*/


    public static int extreme(Complex u,PolygonWrapper P) {
         double test,min;
         int X;
         min=10000.0;
         X=0;
	 Complex v;
         for(int i=0;i<P.count;++i) {
	     v=Complex.times(u,P.z[i]);
	     test=v.y;
            if(test<min) {
             X=i;
	     min=test;
	    }
	 }
	 return(X);
    }



    public static PolygonWrapper cheapHull(PolygonWrapper P) {
	PolygonWrapper Q=new PolygonWrapper();
	int[] n=new int[100];
	Complex u=new Complex();
	for(int i=0;i<32;++i) {
	    double theta=1.0*Math.PI*i/16.0+Math.PI/32.0;
	    u=new Complex(Math.cos(theta),Math.sin(theta));
	    n[i]=extreme(u,P);
	}
	Q.z[0]=P.z[n[0]];
	int ct=1;
	for(int i=1;i<32;++i) {
	   if(Complex.dist(Q.z[ct-1],P.z[n[i]])>.000001) {
	     Q.z[ct]=P.z[n[i]];
	     ++ct;
	   }
	}
	Q.count=ct;
	return(Q);
    }





}