import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.math.*;
import java.util.Arrays;

/**This class does numerical experiments
for triples of polytope which are rectangle
bundles over the (x,P) plane*/

public class PolytopeTripleIntersect {

    public static void printout(int p,int q) {
	Output OUT=new Output("test");

	OUT.writeln("");
	OUT.writeln("");
	OUT.writeln("public class TripleTest {");

	OUT.writeln("public static Polytope P(int k) {");
	for(int k=0;k<210;++k) {
	    Integer K=new Integer(k);
	    OUT.writeln("if(k=="+K.toString()+") return P"+K.toString()+"();");
	}
	OUT.writeln("return null;");
	OUT.writeln("}");

	Polytope[] LIST=new Polytope[1000];
	int count=0;
	boolean test=true;
	PlaidPolytopes MASTER=new PlaidPolytopes();
	double[] U=new double[4];
	while(test==true) {
	    for(int i=0;i<3;++i) {
		U[i]=2*Math.random()-1;
	    }
	    U[3]=Math.random();
	    Polytope[] POLY=MASTER.getTriple(U);
	    Polytope X=intersect(POLY);
	    if(overlap(X,LIST,count)==false) {
		LIST[count]=new Polytope(X);
		System.out.print("polytope "+count+"  ");
	        for(int k=0;k<6;++k) System.out.print(X.code[k]);
		System.out.println("");
		X.javaPrint(OUT,count);
		++count;
	    }
	}
    }

    public static boolean overlap(Polytope P,Polytope[] LIST,int count) {
	for(int i=0;i<count;++i) {
	    if(Polytope.cheapOverlap(P,LIST[i])==true) return(true);
	}
	return false;
    }




    public static Polytope triple(int p,int q,int i,int j) {
	Complex z=new Complex(i+.5,j+.5);
	double[] U=Torus.torusPoint(p,q,z);
	PlaidPolytopes X=new PlaidPolytopes();
	Polytope[] POLY=X.getTriple(U);
	return intersect(POLY);
    }

    /**Numerical experimentation suggests that all the
       vertices of the intersections we are trying to
       find have rational vertices whose denominators
       are in the set {1,2,3,5}.  This routine first
       tests all such points for inclusion in the
       intersection and then takes the convex hull.
       The convex hull algorithm is a random one, and
       is not guaranteed to work.*/

    public static Polytope intersect(Polytope[] POLY) {
	if((POLY[1].type[1]==0)&&(POLY[1].type[2]==0)) return POLY[1];
	Polytope X=new Polytope();
	int count=0;
	double[] d0=cloud0();
	double[] d1=cloud1();
	double[] d2=cloud1();
	double[] d3=cloud3();

	for(int i0=0;i0<d0.length;++i0) {
	for(int i1=0;i1<d1.length;++i1) {
	for(int i2=0;i2<d2.length;++i2) {
	for(int i3=0;i3<d3.length;++i3) {
	    Vector4 T=new Vector4(d0[i0],d1[i1],d2[i2],d3[i3]);
	    boolean test1=contains(T,POLY);
	    if(test1==true) {
		X.V[count]=T;
		++count;
	    }
	}}}}
	X.count=count;
	X=cheapHull(X);	
        X.code[0]=POLY[0].type[1];
	X.code[1]=POLY[0].type[2];
	X.code[2]=POLY[1].type[1];
	X.code[3]=POLY[1].type[2];
	X.code[4]=POLY[2].type[1];
	X.code[5]=POLY[2].type[2];
	return(X);
    }


    /**This routine projects the intersection into the (x,P)
       plane.*/

    public static PolygonWrapper baseProjection(Polytope[] POLY) {

	Polytope X=intersect(POLY);
	PolygonWrapper Y=new PolygonWrapper();
	for(int i=0;i<X.count;++i) {
	    Y.z[i]=new Complex(X.V[i].x[0],X.V[i].x[3]);
	}
	Y.count=X.count;
	Y=PolygonWrapper.cheapHull(Y);
	return(Y);
    }


    /**cheap way of taking the convex hull - not guaranteed.*/

    public static Polytope cheapHull(Polytope P) {
	int count=0;
	Polytope Q=new Polytope();
	for(int i=0;i<10000;++i) {
	    Vector4 W=randomExtreme(P);
	    Q.count=count;
	    boolean test=onList(W,Q);
	    if(test==false) {
		Q.V[count]=new Vector4(W);
		++count;
	    }
	}
	return Q;
    }


    /**gets a random extreme point*/

    public static Vector4 randomExtreme(Polytope P) {
	int index=0;
	Vector4 W=new Vector4();
	for(int i=0;i<4;++i) W.x[i]=1-2*Math.random();
	double max=-10000;
	for(int i=0;i<P.count;++i) {
	    double test=Vector4.dot(P.V[i],W);
	    if(max<test) {
		max=test;
		index=i;
	    }
	}
	return(P.V[index]);
    }

    /**Checks numerically when a vector already is a
       vertex of the polytope.*/

    public static boolean onList(Vector4 W,Polytope P) {
	for(int i=0;i<P.count;++i) {
	    if(Vector4.dist2(W,P.V[i])<.000001) return true;
	}
	return(false);
    }

    /**This checks triple intersections*/
    public static boolean contains(Vector4 V,Polytope[] POLY) {
	for(int i=0;i<2;++i)
	if(POLY[i].contains(V)==false) return false;
	return(true);
    }

    /**This gets the range of points with denominators in {1,2,3,5}*/


    /**parameter coordinate*/
    public static double[] cloud3() {
	double[] d={0,1.0/2,1.0/3,2.0/3,1.0/5,2.0/5,3.0/5,4.0/5,1};
	Arrays.sort(d);
	return(d);
    }

    /**x coordinate*/
    public static double[] cloud0() {
	double[] d1=cloud3();
	double[] d2=new double[49];
	int count=0;
	for(int i=0;i<8;++i) {
	    for(int j=0;j<6;++j) {
		d2[count]=d1[i]+(j-3);
		++count;
	    }
	}
	d2[48]=3;
	Arrays.sort(d2);
	return(d2);
    }

    /**y coordinate*/
    public static double[] cloud1() {
	double[] d1=cloud3();
	double[] d2=new double[17];
	int count=0;
	for(int i=0;i<8;++i) {
	    for(int j=0;j<2;++j) {
		d2[count]=d1[i]+(j-1);
		++count;
	    }
	}
	d2[16]=1;
	Arrays.sort(d2);
	return(d2);
    }

}


