import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.math.*;

public class GraphPETSlicer extends ScaleCanvas implements MouseListener,MouseMotionListener,KeyListener {
    Manager M;
    int X,Y;
    double[] S=new double[3];
    double PAR;
    ControlPanel DIRECTION,LOCK,PARTITION;
    Path2D.Double[] SLICE=new Path2D.Double[3];
    ListenSquare CONTROL;
    ListenSquare GETPAR;
    Lever TILE;

    public GraphPETSlicer() {
	addMouseListener(this);
	addMouseMotionListener(this);
	addKeyListener(this);
	CONTROL=new ListenSquare(0,0,400,70);
	GETPAR=new ListenSquare(310,6,20,20);
	setScales(80,267,190);
	setPanels();
	PAR=2.0/9;
	double t=.5+1.0/9;
	S[0]=.5;
	S[1]=t;
	S[2]=t;
	TILE=new Lever(230,20,2,4);

    }

    public double getSlice() {
	int mode=DIRECTION.mode;
	return(S[mode]);
    }

    public double getParameter() {
	return PAR;
    }

    public void setPanels() {
         Color[] C0={new Color(80,140,255),
         Color.white,Color.white,Color.black,Color.red};
	 Color c=Color.white;

	 String[] DirectionString={
	     "X","Y","Z","slice"};
	 int[] DirectionState={0,0,1};
         DIRECTION=new ControlPanel(C0,DirectionString,DirectionState,3);
	 DIRECTION.mode=2;

	 String[] LockString={"locked","free","parameter"};
	 int[] LockState={1,0};
         LOCK=new ControlPanel(C0,LockString,LockState,2);
	 LOCK.mode=0;

	 String[] DynamicsString={"(+)","(--)","partition"};
	 int[] DynamicsState={1,0};
         PARTITION=new ControlPanel(C0,DynamicsString,DynamicsState,2);
	 PARTITION.mode=0;
    }




    public Path2D.Double makeSlider(double s,double a,double w) {
	Path2D.Double gp=new Path2D.Double();
	gp.moveTo(0,0);
	gp.lineTo(w,0);
	gp.lineTo(w,1);
	gp.lineTo(0,1);
	gp.closePath();
	gp.moveTo(s,-1);
	gp.lineTo(s,2);
	gp.moveTo(-2,a);
	gp.lineTo(3,a);
	return(gp);
    }



   public void paint(Graphics g2) {
      Graphics2D g=(Graphics2D) g2;
      g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                         RenderingHints.VALUE_ANTIALIAS_ON);
      drawSlicer(g);
      if(M.C.GRAPH.L[2].on==1) drawTriple(g);
      if(M.C.GRAPH.L[3].on==1) drawTriple(g);  
      drawControls(g);
   }

    public void drawSlicer(Graphics2D g) {
	g.setColor(new Color(0,100,200));
	g.fillRect(0,0,getWidth(),getHeight());
	int p=M.p();
	int q=M.q();
	double A=1.0*p/q;
	double[] w={1,1+A,1+A};
	int c=DIRECTION.mode;
	Path2D.Double gp=makeSlider(S[c],PAR,w[c]);
	gp=transform(gp);
	g.setColor(new Color(200,0,200));
	g.fill(gp);
	g.setColor(Color.white);
	g.draw(gp);
    }

    public void drawControls(Graphics2D g) {
	CONTROL.render(g,new Color(0,0,100));
	DIRECTION.render(g,0,0,45);
	LOCK.render(g,50,0,75);
	PARTITION.render(g,130,0,75);
	GETPAR.render(g,new Color(255,0,255));
	TILE.render(g,"tiling extent",Color.blue);
	g.setColor(Color.white);
	g.drawString("grab",338,12);
	g.drawString("parameter",338,27);
    }

    public void grabParameter() {
	int p=M.p();
	int q=M.q();
	PAR=1.0*p/q;
    }

    public void setProjectionCenter() {
	PolygonWrapper Q=getProjection();
	if(Q==null) return;
	Complex z=Q.center();
        S[2]=z.x; 
        PAR=z.y;
    }

    public void drawTriple(Graphics2D g) {
	PolygonWrapper Q=getProjection();
	if(Q==null) return;
	Path2D.Double gp=Q.toPath2D();
	gp=transform(gp);
	g.setColor(new Color(255,255,255,100));
	g.fill(gp);
	g.setColor(Color.black);
	g.draw(gp);
    }

    public PolygonWrapper getProjection() {
	for(int i=0;i<218;++i) {
	    Polytope P=AllPolytopes.graphTriple(i);
	    boolean test=false;
	    try{test=M.Z.TS.matchesCode(P,i);}
	    catch(Exception e) {}
	    if(test==true) {
		PolygonWrapper Q=project(P);
		return(Q);
	    }
	}
	return null;
    }

    public PolygonWrapper project(Polytope P) {
        int s=DIRECTION.mode;
	PolygonWrapper Q=new PolygonWrapper();
	Q.count=P.count;
	for(int i=0;i<P.count;++i) {
	    Q.z[i]=new Complex(P.V[i].x[s],P.V[i].x[3]);
	}
	Q=PolygonWrapper.cheapHull(Q);
	return(Q);
    }

    public void documentStuff(Point X) {
	if(DIRECTION.L[20].inside(X)==1) DocumentGraph.slice(M.D);
	if(LOCK.L[20].inside(X)==1) DocumentGraph.lock(M.D);
	if(PARTITION.L[20].inside(X)==1) DocumentGraph.partition(M.D);
	if(TILE.L[4].inside(X)==1) DocumentGraph.tiling(M.D);
    }




    public void mouseClicked(MouseEvent e) {
	MouseData J=MouseData.process(e);
	DIRECTION.switchMode(J.X);
	PARTITION.switchMode(J.X);
	TILE.process(J.X);
	LOCK.switchMode(J.X);
	if(GETPAR.inside(J.X)==1) grabParameter();
	if(CONTROL.inside(J.X)==0) {
          if(J.mode==1)  scaleUp(J.X,0);
          if(J.mode==2)  {
	      int c=DIRECTION.mode;
              Complex z=unTransform(J.X);
	      S[c]=z.x;
	      if((LOCK.mode==1)&&(z.y>0)&&(z.y<1)) PAR=z.y;
	  }
          if(J.mode==3)  scaleUp(J.X,1);
	}
	documentStuff(J.X);
	M.repaint();
    }

    public void mouseDragged(MouseEvent e) {
	MouseData J=MouseData.process(e);
	if(CONTROL.inside(J.X)==0) {
	  int c=DIRECTION.mode;
          Complex z=unTransform(J.X);
	  S[c]=z.x; 
          if((LOCK.mode==1)&&(z.y>0)&&(z.y<1)) PAR=z.y;
	}
	repaint();
	M.G.repaint();
    }


    public void mousePressed(MouseEvent e) {}

    public void mouseReleased(MouseEvent e) {}
    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}   
    public void mouseMoved(MouseEvent e) {}


    public void keyTyped(KeyEvent e) {}
    public void keyPressed(KeyEvent e) {}
    public void keyReleased(KeyEvent e) {}

}


