import java.awt.*;
import java.math.*;


public class PlaidClassifyingMap {

    public static double dec1(double x) {
	return(x-Math.floor(x));
    }

    /**range is [-1,1]*/

    public static double dec2(double x) {
	double y1=x/2.0+1/2.0;
	double y2=dec1(y1);
	double y3=2*y2-1;
	return(y3);
    }

    public static double dec4(double x) {
	double y1=x/4.0+1/2.0;
	double y2=dec1(y1);
	double y3=4*y2-2;
	return(y3);
    }

    /**the maps*/

    public static double[] map(double P,double x, double y) {
	double a1=2*P*x+2*y;
	double a2=2*P*x;
	double a3=2*P*x+2*P*y;
	double[] X={P,a1,a2,a3};
	return(X);
    }


    public static double[] mapEnhanced(double P,Complex Z) {
	return mapEnhanced(P,Z.x,Z.y);
    }

    public static double[] mapBasic(double P,Complex Z) {
	return mapBasic(P,Z.x,Z.y);
    }


    /**This does the map for the directed model*/

    public static double[] mapEnhanced(double P,double x, double y) {
	double[] X=map(P,x,y);
	X=reduce(P,X);
	return(X);
    }


    /**This is an alternate version of the enhanced map.
       It is more of a local calculation*/

    public static double[] mapEnhancedAlt(double P,double x, double y) {
	double t=dec4(2*P*x+2*y);
	double b=0.5*P*t;
	double u1=dec2(P*(2-P)*x+b-P*y);
	double u2=dec2(P*(2-P)*x+b+P*y);
	double[] X={t,u1,u2,P};
	return(X);
    }


    /**This does the map for the undirected model.*/

    public static double[] mapBasic(double P,double x, double y) {
	double[] X=map(P,x,y);
	X=reduce2(P,X);
	return(X);
    }


    /**The parameter is the last coordinate*/

    public static double[] reduce(double P,double[] u) {
	double x=u[1]/4;
	x=Math.floor(x+.5);
	x=4*x;
	double[] v={0,0,0,0};
	v[0]=u[1]-x;
	v[1]=u[2]-x*P/2;
	v[2]=u[3]-x*P/2;
	v[1]=dec2(v[1]);
	v[2]=dec2(v[2]);
	v[3]=u[0];
	return(v);
    }

    /**The parameter is the last coordinate*/

    public static Vector4 reduce2(Vector4 V) {
	double P=V.x[3];
	double[] u={P,V.x[0],V.x[1],V.x[2]};
	double[] v=reduce2(P,u);
	Vector4 W = new Vector4(v[0],v[1],v[2],P);
	return W;
    }

    public static double[] reduce2(double P,double[] u) {
	double x=u[1]/2;
	x=Math.floor(x+.5);
	x=2*x;
	double[] v={0,0,0,0};
	v[0]=u[1]-x;
	v[1]=u[2]-x*P/2;
	v[2]=u[3]-x*P/2;
	v[1]=dec2(v[1]);
	v[2]=dec2(v[2]);
	v[3]=u[0];
	return(v);
    }



}

