import java.awt.*;
import java.math.*;

/**This file converts from the plaid classifying map
the the arithmetic graph classifying map.  The main
issue is getting the first coordinate.*/

public class PlaidToGraph {

    /**This is the main routine - for polytopes.  This only
works correctly when the interior of the polytope is
contained in a single domain of definition/continuity for
the map.*/

    public static Polytope convert(Polytope P) {
	Polytope Q=convertPure(P);
	Q=GraphPolytopeMoves.adjustByShifting(Q);
	for(int i=0;i<6;++i) Q.code[i]=P.code[i];
	return(Q);
    }


    public static Polytope convertPure(Polytope P) {
	Polytope Q=new Polytope();
	Q.count=P.count;
	Vector4 V=P.center();
	double[] U={V.x[0],V.x[1],V.x[2],V.x[3]};
	int r=getRegion(U);
	for(int i=0;i<P.count;++i) Q.V[i]=convert(P.V[i],r);
	for(int i=0;i<6;++i) Q.code[i]=P.code[i];
	return(Q);
    }


    public static Vector4 convert(Vector4 V,int r) {
	double[] U={V.x[0],V.x[1],V.x[2],V.x[3]};
	double[] X=convert(U,r);
	double A=U[3]/(2-U[3]);
	Vector4 W=new Vector4(X[0],X[1],X[2],A);
	return(W);
    }

    public static Vector4 convert(Vector4 V) {
	double[] U={V.x[0],V.x[1],V.x[2],V.x[3]};
	int r=getRegion(U);
	return convert(V,r);
    }


    public static double[] convert(double[] U) {
	int r=getRegion(U);
	return convert(U,r);
    }

    /**Here is the main conversion map.  The map does different
       things when r=0 and r=1. */

    public static double[] convert(double[] U,int r) {
	double[] d={U[0]-U[1],-U[1]-1,U[2]+U[3]+1};
	double e=-U[3]+2;
	for(int i=0;i<3;++i) d[i]=d[i]/e;
	d[0]=d[0]+r;
	return d;
    }

    /**This partitions the xy plane into 2 regions*/

    public static int getRegion(Vector4 V) {
	double[] U={V.x[0],V.x[1],V.x[2],V.x[3]};
	return getRegion(U);
    }

    public static int getRegion(double[] U) {
	if(U[0]<U[1]) return 1;
	return -1;
    }

}



