import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.math.*;

/**This class checks that the plaid triple partition really is
the partition we claim it is.  First we show that each intersection
really is contained in the pieces it should be. Then we check that
the pieces have pairwise disjoint interiors. Finally, we check by
computing volumes exactly that the triple partition pieces exactly
fill theor supposed container*/

public class ProofTripleCombinatorics {


    public static void main() {
	testContainer();
	testCombinatorics();
    }

    /**This checks that all the pieces of the triple partition
       lie in the set [-1,1]^3 x [0,1]. Since we have scaled by
       60 we really are testing the larger container
       [-60,60]^3 x [0,60]*/

    public static void testContainer() {
	System.out.print("start container test ");
	for(int i=0;i<218;++i) {
	    PolytopeLong P=AllPolytopesLong.plaidTriple(i);
	    for(int j=0;j<P.count;++j) {
		for(int k=0;k<3;++k) {
		    if(P.V[j].x[k]<-60) throw new ProofException("out of bounds");
		    if(P.V[j].x[k]>60) throw new ProofException("out of bounds");
		}
		if(P.V[j].x[3]<0)  throw new ProofException("out of bounds");
		if(P.V[j].x[3]>60)  throw new ProofException("out of bounds");
	    }
	}
	System.out.println("done");
    }





    /**This does the containment test below for every polytope
       in the triple partition*/

    public static void testCombinatorics() {
	for(int dir=0;dir<3;++dir) {
	  System.out.print("start contain test "+dir+" ");
	  for(int i=0;i<218;++i) {
	    PolytopeLong P=AllPolytopesLong.plaidTriple(i);
	    boolean test=testCombinatorics(dir,i,P);
	    if(test==false) throw new ProofException("fail");
	  }
	System.out.println("done");
	}
    }


    /**This tests whether a plaid triple polytope is contained in 
       a plaid (single) polytope of the correct type. There are
       3 tests, for each of the three partitions:
           
          0. the plaid partition
          1. the image of the plaid partition under the forward curve following map.
          2. the image of the plaid partition under the backward curve following map.

This is part of the proof that each triple is the triple intersection of polytopes,
one from each of the 3 partitions.*/


    public static boolean testCombinatorics(int dir,int k,PolytopeLong P) {
	for(int i0=-2;i0<3;++i0) {
	for(int i1=-2;i1<3;++i1) {
	for(int i2=-2;i2<3;++i2) {
	    for(int j=0;j<26;++j) {
		PolytopeLong Q=AllPolytopesLong.plaid(j);
		Q=AllPolytopesLong.plaidShift(i0,i1,i2,Q);
		Q=AllPolytopesLong.plaidMap(dir,Q);
		if(Q.contains(P)==true) {
		    int[] type=DataPlaidPolytopes.type(j);
		    int[] code=DataTripleCodes.data(k);
		    int[] code0={code[0],code[1]};
		    int[] code1={code[2],code[3]};
		    int[] code2={code[4],code[5]};
		    int[][] code012={code1,code2,code0};
		    return match (type,code012[dir]);
		}
	    }
	}}}
	return false;
    }

    public static boolean match(int[] A,int[] B) {
	if((A[0]==B[0])&&(A[1]==B[1])) return true;
	if((A[0]==B[1])&&(A[1]==B[0])) return true;
	return false;
    }



}



