import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.math.*;

public class Spider {

    public Spider() {}

    /**These routines encode how the shortest (planar AG lattice)
       vectors emanating from a point inside an integer unit square
       cross the edges of the square*/

    public static String codon(int p,int q,Complex z) {
	String S="";
	for(int i=-1;i<2;++i) {
	    for(int j=-1;j<2;++j) {
		if((i!=0)||(j!=0)) {
		    S=S+codon(p,q,z,i,j);
		}
	    }
	}
	return(S);
    }

    public static String codon(int p,int q,Complex z,int i,int j) {
	int n=codon0(p,q,z,i,j);
	Integer N=new Integer(n);
	return N.toString();
    }


    public static int codon0(int p,int q,Complex z,int i,int j) {
	Complex w=AGLattice.shortVector(p,q,i,j);
	w=Complex.plus(z,w);
	double x1=Math.floor(z.x);
	double y1=Math.floor(z.y);
	double x2=Math.floor(z.x)+1;
	double y2=Math.floor(z.y)+1;
	Complex z1=new Complex(x1,y1);
	Complex z2=new Complex(x1,y2);
	Complex z3=new Complex(x2,y2);
	Complex z4=new Complex(x2,y1);
	if(Complex.checkCross(z,w,z1,z4)==true) return 0;
	if(Complex.checkCross(z,w,z2,z1)==true) return 1;
	if(Complex.checkCross(z,w,z3,z2)==true) return 2;
	if(Complex.checkCross(z,w,z4,z3)==true) return 3;
	return(-1);
    }

    /**Here are the spider sets*/

    public static int convert(int i,int j) {
	int[][] K={{0,0},{0,1},{0,2},{1,0},{1,2},{2,0},{2,1},{2,2}};
	for(int k=0;k<8;++k) {
	    if((i==K[k][0])&&(j==K[k][1])) return(k);
	}
	return(-1);
    }


    public static int edge(int i,int j) {
	int[] a={0,1,3,2,0,1,3,2};
	int[] b={1,0,0,1,3,2,2,3};
	if(j==0) return a[i];
	return b[i];
    }

    public static PolygonWrapper getSpider(int i,int j,int f,double A) {
	if((i==1)&&(j==1)) return(null);
	int k=convert(i,j);
	int e=edge(k,f);


	PolygonWrapper P=spider(i,j,f,A);
	P.edge=e;
	return(P);
    }

    public static PolygonWrapper spider(int i,int j,int f,double A) {
	if((i==0)&&(j==0)) return(spider0(f,A));
	if((i==0)&&(j==1)) return(spider1(f,A));
	if((i==0)&&(j==2)) return(spider2(f,A));
	if((i==1)&&(j==0)) return(spider3(f,A));
	if((i==1)&&(j==2)) return(spider4(f,A));
	if((i==2)&&(j==0)) return(spider5(f,A));
	if((i==2)&&(j==1)) return(spider6(f,A));
	if((i==2)&&(j==2)) return(spider7(f,A));
	return null;
    }

    public static Complex[] reflect(double A,Complex[] Z) {
	Complex[] W=new Complex[Z.length];
	for(int i=0;i<W.length;++i) {
	    W[i]=new Complex(); 
            W[i].x=1-Z[i].x;
	    W[i].y=2*A+1-Z[i].y;
	}
	return(W);
    }



    public static PolygonWrapper toPolygon(Complex[] Z) {
	PolygonWrapper P=new PolygonWrapper(Z.length,Z);
	return P;
    }

    /**Here is the data for the spider regions.*/

    public static Complex[] spiderList00(double A) {
	Complex[] Z=new Complex[3];
	Z[0]=new Complex(0,0);
	Z[1]=new Complex(1,1);
	Z[2]=new Complex(1,A);
	return(Z);
    }

    public static Complex[] spiderList01(double A) {
	Complex[] Z=spiderList00(A);
	Complex[] W=reflect(A,Z);
	Complex[] B={W[0],W[2],Z[0],Z[1]};
	return(B);
    }

    public static Complex[] spiderList10(double A) {
	Complex[] Z=new Complex[3];
	double x=(A+A*A)/(1+2*A-A*A);
	double y=(1+3*A+2*A*A)/(1+2*A-A*A);
	Z[0]=new Complex(0,0);
	Z[1]=new Complex(x,y);
	Z[2]=new Complex(0,1+A);
	return(Z);
    }

    public static Complex[] spiderList11(double A) {
	Complex[] Z=spiderList10(A);
	Complex[] W=reflect(A,Z);
	Complex[] B={W[0],W[2],Z[0],Z[1]};
	return(B);
    }

    public static Complex[] spiderList20(double A) {
	Complex[] Z=new Complex[3];
	double x=4*A/(1+4*A-A*A);
	double y=(1+5*A+7*A*A-A*A*A)/(1+4*A-A*A);
	Z[0]=new Complex(1,A);
	Z[1]=new Complex(x,y);
	Z[2]=new Complex(1,1+2*A);
	return(Z);
    }

    public static Complex[] spiderList21(double A) {
	Complex[] Z=spiderList20(A);
	Complex[] W=reflect(A,Z);
	Complex[] B={W[0],W[2],Z[0],Z[1]};
	return(B);
    }
    public static Complex[] spiderList30(double A) {
	Complex[] Z=new Complex[3];
	Z[0]=new Complex(0,1+A);
	Z[1]=new Complex(1,1);
	Z[2]=new Complex(1,1+2*A);
	return(Z);
    }


    public static Complex[] spiderList31(double A) {
	Complex[] Z=spiderList30(A);
	Complex[] W=reflect(A,Z);
	Complex[] B={W[0],W[2],Z[0],Z[1]};
	return(B);
    }


    public static Complex[] spiderList0(int f,double A) {
	if(f==0) return spiderList00(A);
	return spiderList01(A);
    }

    public static Complex[] spiderList1(int f,double A) {
	if(f==0) return spiderList10(A);
	return spiderList11(A);
    }

    public static Complex[] spiderList2(int f,double A) {
	if(f==0) return spiderList20(A);
	return spiderList21(A);
    }

    public static Complex[] spiderList3(int f,double A) {
	if(f==0) return spiderList30(A);
	return spiderList31(A);
    }

    public static PolygonWrapper spider0(int f,double A) {
	Complex[] Z=spiderList0(f,A);
	return toPolygon(Z);
    }

    public static PolygonWrapper spider1(int f,double A) {
	Complex[] Z=spiderList1(f,A);
	return toPolygon(Z);
    }

    public static PolygonWrapper spider2(int f,double A) {
	Complex[] Z=spiderList2(f,A);
	return toPolygon(Z);
    }

    public static PolygonWrapper spider3(int f,double A) {
	Complex[] Z=spiderList3(f,A);
	return toPolygon(Z);
    }

    public static PolygonWrapper spider4(int f,double A) {
	Complex[] Z=spiderList3(f,A);
	Z=reflect(A,Z);
	return toPolygon(Z);
    }

    public static PolygonWrapper spider5(int f,double A) {
	Complex[] Z=spiderList2(f,A);
	Z=reflect(A,Z);
	return toPolygon(Z);
    }

    public static PolygonWrapper spider6(int f,double A) {
	Complex[] Z=spiderList1(f,A);
	Z=reflect(A,Z);
	return toPolygon(Z);
    }

    public static PolygonWrapper spider7(int f,double A) {
	Complex[] Z=spiderList0(f,A);
	Z=reflect(A,Z);
	return toPolygon(Z);
    }


    /**Here is the routine which converts an affine functional into
       a line in the xy plane.  The affine functionals are from the
       avoidance proofs. Here the xy plane is identified with the
       fiber over the point (z,A).  The last coordinate is scaled
       down by 60 because, in our proofs, we use affine functionals
       that have been scaled up by 60 to clear denominators.*/



    public static Path2D.Double makeBarrier(double A,long[] L) {
	Path2D.Double gp=new Path2D.Double();

	double C=A*L[3]+L[4]/60.0;



	if(L[1]!=0) {
   	    double x1=-4;
	    double x2=+4;
	    double y1=(-x1*L[0]-C)/L[1];
	    double y2=(-x2*L[0]-C)/L[1];
	    gp.moveTo(x1,y1);
	    gp.lineTo(x2,y2);
	    return(gp);
	}

	double x=-C/L[0];
	gp.moveTo(x,-4);
	gp.lineTo(x,+4);
	return(gp);
    }

    /**This makes a list of such objects*/

    public static Path2D.Double[] makeBarrier(double A,long[][] L) {
	Path2D.Double[] gp=new Path2D.Double[L.length];
	for(int i=0;i<L.length;++i) gp[i]=makeBarrier(A,L[i]);
	return gp;
    }




}